/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Cherokee
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2001-2002 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include "plugin_table.h"
#include "handler.h"

ret_t
cherokee_plugin_table_new  (cherokee_plugin_table_t **pt)
{
	   cherokee_plugin_table_t *n = (cherokee_plugin_table_t *)malloc(sizeof(cherokee_plugin_table_t));

	   cherokee_table_init (&n->table);
	   cherokee_table_init (&n->handlers);

	   *pt = n;
	   
	   return ret_ok;
}


ret_t
cherokee_plugin_table_free (cherokee_plugin_table_t *pt)
{
	   cherokee_table_clean (&pt->table);
	   cherokee_table_clean (&pt->handlers);

	   free (pt);

	   return ret_ok;
}


void *
cherokee_plugin_table_get_from_url (cherokee_plugin_table_t *pt, char *dir)
{
	   void *ret = NULL;
	   int   length;
	   char *slash;
	   char *url;

	   url = strdup(dir);

	   do {
			 ret = cherokee_table_get (&pt->table, url);

			 /* Found
			  */
			 if (ret != NULL)
				    goto go_out;

			 length = strlen(url);
			 if (length == 0)
				    goto go_out;

			 /* Modify url for next loop
			  */
			 if (url[length-1] == '/') {
				    url[length-1] = '\0';
			 } else {
				    slash = rindex (url, '/');
				    slash[1] = '\0';
			 }
	   } while (ret == NULL);


go_out:
	   free (url);
	   return ret;
}


static inline func_new_t
load_plugin (char *plugin_name)
{
	void       *lib;
	char        tmp[0xFF];
	func_new_t  func_new;
	void      (*func_init) (void);
	int         dl_flags;
	
	/* Load the library
	 */	
#ifdef HAVE_RTLDNOW
	dl_flags = RTLD_NOW;
#else
	dl_flags = RTLD_LAZY;
#endif

#ifdef HAVE_RTLDGLOBAL
	dl_flags |= RTLD_GLOBAL;
#endif

	snprintf (tmp, 0xFF, "%s/libcherokee_%s." SO_SUFFIX, CHEROKEE_PLUGINDIR, plugin_name);
	lib = dlopen (tmp, dl_flags);
	if (lib == NULL) {
		fprintf (stderr, "%s\n", dlerror());
		return NULL;
	}
	

	/* Maybe execute the library init function
	 */
	snprintf (tmp, 0xFF, "%s_init", plugin_name);
	func_init = dlsym(lib, tmp);
	if (func_init != NULL) {
		func_init();
	}


	/* Get the constructor
	 */
	snprintf (tmp, 0xFF, "cherokee_handler_%s_new", plugin_name);
	func_new = (func_new_t) dlsym(lib, tmp);
	if (func_new == NULL) {
		fprintf (stderr, "%s\n", dlerror());
		return NULL;
	}
	
	return func_new;
}


ret_t 
cherokee_plugin_table_set (cherokee_plugin_table_t *pt, char *dir, char *handler)
{
	   ret_t      ret;
	   func_new_t func;

	   /* Get the "handler"_new function
	    */
	   func = (func_new_t) cherokee_table_get (TABLE(&pt->handlers), handler);
	   if (func == NULL) {
			 func = load_plugin (handler);
			 if (func == NULL) {
				    return ret_error;
			 }

			 /* Add to table
			  */
			 ret = cherokee_table_add (&pt->handlers, handler, (void*)func); 
			 if (ret != ret_ok) {
				    return ret_error;
			 }
	   }
	   

	   /* Add "handler" <-> handler_new func
	    */
	   cherokee_table_add (TABLE(&pt->table), dir, (void*) func);

	   return ret_ok;
}

